"""
Set up and run the PyWSGIRef server with a simple HTML response.
"""
from PyWSGIRef import *

def main():
    """
    Main function to set up and run the PyWSGIRef server.
    """
    # define main html response
    MAIN_HTML = """<!DOCTYPE html>
    <html lang="de">
        <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <title>PyWSGIRef Server</title>
        </head>
        <body>
            <h1>Willkommen zum PyWSGIRef Server!</h1>
            <p>Dies ist die Hauptseite des Servers.</p>
            <p>Sie nutzen Version {} von PyWSGIRef.</p>
        </body>
    </html>"""

    # add Schablone 'Hallo Welt' as main
    addSchablone("main", MAIN_HTML)

    # set up application object
    def contentGenerator(path: str) -> str:
        """
        Serves as the main WSGI application.
        """
        match path:
            case "/":
                content = SCHABLONEN["main"].decoded().format(about()["Version"])
            case "/hello":
                content = HELLO_WORLD
            case _:
                content = ERROR
        return content

    # make the application object
    application = makeApplicationObject(contentGenerator)

    # set up server
    server = setUpServer(application)
    server.serve_forever()

    # Note: This code is intended to be run as a script, not as a module.
    print("Successfully started WSGI server on port 8000.")

if __name__ == "__main__":
    main()


HELLO_WORLD = """<!DOCTYPE html>
<html lang"de">
    <body>
        <h1>Hallo Welt!</h1>
    </body>
<html>"""

ERROR = """<!DOCTYPE html>
<html lang"de">
    <body>
        <h1>Fehler!</h1>
        <p>Es ist ein Fehler aufgetreten.</p>
    </body>
</html>"""


"""
PYWSGIREF
"""
from typing import Callable
import requests
from wsgiref.simple_server import make_server, WSGIServer

from .exceptions import *
from .pyhtml import PyHTML
from .defaults import *

def about():
    """
    Returns information about your release and other projects by LK
    """
    return {"Version":(1, 0, 1), "Author":"Leander Kafemann", "date":"07.06.2025",\
            "recommend":("Büro by LK",  "pyimager by LK", "naturalsize by LK"), "feedbackTo": "leander@kafemann.berlin"}

SCHABLONEN = {}
finished = False

def loadFromWeb(url: str, data: dict = {}) -> str:
    """
    Loads content from the given URL with the given data.
    """
    if finished:
        raise ServerAlreadyGeneratedError()
    if not url.endswith(".pyhtml"):
        raise InvalidFiletypeError()
    rq = requests.post(url, data).content
    return rq.decode()

def loadFromFile(filename: str) -> str:
    """
    Loads a file from the given filename.
    """
    if finished:
        raise ServerAlreadyGeneratedError()
    if not filename.endswith(".pyhtml"):
        raise InvalidFiletypeError()
    with open(filename, "r", encoding="utf-8") as f:
        content = f.read()
    return content

def addSchablone(name: str, content: str):
    """
    Adds a template to the SCHABLONEN dictionary.
    """
    global SCHABLONEN
    SCHABLONEN[name] =  PyHTML(content)

def makeApplicationObject(contentGeneratingFunction: Callable) -> Callable:
    """
    Returns a WSGI application object based on your contentGeneratingFunction.
    The contentGeneratingFunction should take a single argument (the path) and return the content as a string.
    """
    if not callable(contentGeneratingFunction):
        raise InvalidCallableError()
    def simpleApplication(environ, start_response) -> list:
        """
        A simple WSGI application object that serves as a template.
        """
        type_ = "text/html" 
        status = "200 OK"
        content = contentGeneratingFunction(environ["PATH_INFO"])
        headers = [("Content-Type", type_),
                   ("Content-Length", str(len(content))),
                   ('Access-Control-Allow-Origin', '*')]
        start_response(status, headers)
        return [content.encode("utf-8")]
    return simpleApplication

def setUpServer(application: Callable) -> WSGIServer:
    """
    Creates a WSGI server.
    No additional Schablonen can be loaded from the web.
    """
    global finished
    finished = True
    server = make_server('', 8000, application)
    return server


class ServerAlreadyGeneratedError(Exception):
    """
    Raised when a server is already generated,
    so that further modifications are not allowed.
    """
    def __init__(self, message="Server has already been generated."):
        self.message = message
        super().__init__(self.message)

class InvalidFiletypeError(Exception):
    """
    Raised when an invalid file type (not PyHTML) is encountered.
    """
    def __init__(self, message="Invalid file type. Only .pyhtml files are allowed."):
        self.message = message
        super().__init__(self.message)

class InvalidCallableError(Exception):
    """
    Raised when a callable is expected but not provided.
    """
    def __init__(self, message="Expected a callable object."):
        self.message = message
        super().__init__(self.message)